{% extends "main.html" %}

{% block content %}
<div align="center">
  <h1>404 - Not found</h1>
  <img src="/assets/boink.jpg" alt="Black Niji" width="300">
  <p>There's nothing to see here!</p>
  <a href="/">Go Home.</a>
</div>
{% endblock %}
