h1 {
	background: linear-gradient(
		45deg,
		#e3655b 0%,
		/* warm pink/red */ #db5461 40%,
		/* soft peach */ #ee99a0 75%,
		/* calm blue */ #363a4f 100% /* deeper blue */
	);
	background-clip: text;
	-webkit-background-clip: text;
	-webkit-text-fill-color: transparent;
}

h2 {
	background: -webkit-linear-gradient(10deg, #bd34fe 5%, #e43498 15%);
	background-clip: text;
	-webkit-background-clip: text;
	-webkit-text-fill-color: transparent;
}

h3 {
	background: linear-gradient(
		45deg,
		#34d1fe 0%,
		/* bright cyan */ #34fe9e 50%,
		/* mint green */ #fef534 100% /* sunny yellow */
	);
	background-clip: text;
	-webkit-background-clip: text;
	-webkit-text-fill-color: transparent;
}

.flameshot img {
	border-radius: 35px;
	border: 3px solid #000d1a;
}
